<?php



require_once 'Classes/Model/Observation.php';
require_once 'Classes/Pulsar/ObservationData2.php';



/**
 * @class ObservationSet
 *
 * This class represents an observation and a set of associated calibrations. These
 * calibrations will be within X minutes of the observation
 **/
class ObservationSet
{
  // PUBLIC

  public function __construct( ObservationSet $rhs = null )
  {
    $this->observation = null;
    $this->calibrations = null;
    $this->observation_data = null;
    $this->data_dir = Dir::ConvertPathString( '/pulsar/scratch01/data_archive/' );

    if( $rhs != null )
      $this->Copy( $rhs );
  }
  public function Copy( ObservationSet $rhs )
  {
    $this->observation = new Observation( $rhs->GetObservation() );

    $this->calibrations = Array();
    foreach( $rhs->calibrations as $next_cal )
      $this->calibrations[] = new Calibration( $next_cal );

    $this->observation_data = new ObservationData2( $rhs->observation_data );
  }

  public function SetObservation( $s_observation ) { $this->observation = $s_observation; }
  public function GetObservation() { return $this->observation; }

  public function SetCalibrations( $s_calibrations ) { $this->calibrations = $s_calibrations; }
  public function GetCalibrations() { return $this->calibrations; }
  
  //! Get the adjusted filename, this will include scrunching parameters if set
  //public function GetDataFile() { return $this->priv_GetDataFile(); }
  
  public function SetScrunchingParameters( ScrunchingParameters $s_scr_params ) { $this->priv_SetScrunchingParameters( $s_scr_params ); }
  
  //public function GetScrunchingParameters() { return $this->observation_data->GetScrunchingParameters(); }
  
  public function SetupObservationData() { $this->priv_SetupObservationData(); }
  
  public function EstimateDownloadableSize() { return $this->priv_EstimateDownloadableSize(); }
  
  public function GetSourceFile() { $this->priv_GetSourcefile(); }
//   
//   public function MakeDownloadables() { $this->priv_MakeDownloadables(); }
  
  //! Get the obs file, this will be in the data directory and be added/scrunched etc as required
  public function GetFinalObsFile() { return $this->priv_GetFinalObsFile(); }

  //! Get the cal files, these will be in the data directory and be added if required.
  public function GetFinalCalFiles() { return $this->priv_GetFinalCalFiles(); }
  
  // PRIVATE

  private $observation;
  private $calibrations;
  private $observation_data;
  private $data_dir;
  
  private function priv_SetScrunchingParameters( $s_scr_params )
  {
    $this->observation_data->SetScrunchingParameters( $s_scr_params );
  }
  
  //! This function creates the observation_data object and sets it up
  private function priv_SetupObservationData()
  {
    $this->observation_data = new ObservationData2();
    $this->observation_data->SetObservationHeader( $this->observation );
    $this->observation_data->SetDataDir( $this->data_dir );
  }
  
  private function priv_EstimateDownloadableSize()
  {
    $dl_size = $this->observation_data->EstimateDownloadableSize();
    
    $estimater = new ObservationData2();
    foreach( $this->calibrations as $next_cal )
    {
      $estimater->SetObservationHeader( $next_cal );
      $dl_size += $estimater->EstimateDownloadableSize();
    }

    return $dl_size;
  }
  
  //! Make the source file, don't need the full downloadable files if we are plotting
  private function priv_GetSourceFile()
  {
    $this->observation_data->GetDataFile();
  }
  
  
  private function priv_MakeDownloadables()
  {
    $this->observation_data->MakeDownloadableFile();
    foreach( $this->calibrations as $next_cal )
    {
      $src_file = $next_cal->GetFile();
      $dest_file = File::ConvertFullPath( $this->data_dir->GetPath() . '/' . $src_file->GetFilename() );
      if( !$dest_file->Exists() )
      {
        if( $src_file->Exists() && !$src_file->IsDir() )
        {
          symlink( $src_file->GetFullPath(), $dest_file->GetFullPath() );
        }
        else
        {
          $new_data = new ObservationData2();
          $new_data->SetObservationHeader( $next_cal );
          $new_data->SetDataDir( $this->data_dir );
          $new_data->MakeDownloadableFile();
        }
      }
    }
  }
  
  private function priv_GetDownloadFiles()
  {
    $data_files = Array();
    $data_files[] = $this->observation_data->GetDataFile();
    foreach( $this->calibrations as $next_cal )
    {
      $cal_data_path = $this->data_dir->GetPath();
      $cal_data_path .= '/' . $next_cal->GetFile()->GetFilename();
      $data_files[] =  File::ConvertFullPath( $cal_data_path );
    }
    return $data_files;
  }
  
  private function priv_GetFinalObsFile()
  {
    if( $this->observation_data != null )
      $file = new File( $this->observation_data->GetDataFile() );
    else
      $file = new File( $this->observation->GetFile() );
    
    return $file;
  }
  
  private function priv_GetFinalCalFiles()
  {
    $data_files = Array();
    foreach( $this->calibrations as $next_cal )
    {
      $cal_data_path = $this->data_dir->GetPath();
      $cal_data_path .= '/' . $next_cal->GetFile()->GetFilename();
      $data_files[] =  File::ConvertFullPath( $cal_data_path );
    }
    return $data_files;
  }

};

